@@ -1,4 +1,4 @@
 from .cubing import cubing
-from .downsampling import downsample_mag
+from .downsampling import downsample_mags
 from .compress import compress_mag
 from .metadata import write_webknossos_metadata
