@@ -1,17 +1,19 @@
 import operator
+from argparse import Namespace
 from shutil import rmtree
 from abc import ABC, abstractmethod
 from os import makedirs, path
 from os.path import join, normpath, basename
 from pathlib import Path
-from typing import Type, Tuple, Union, Dict, Any, Optional, cast
+from typing import Type, Tuple, Union, Dict, Any, Optional, cast, TypeVar, Generic
 
 import numpy as np
 import os
 import re
 
+from wkcuber.api.bounding_box import BoundingBox
 from wkcuber.mag import Mag
-from wkcuber.utils import logger
+from wkcuber.utils import logger, get_executor_for_args, ceil_div_np
 
 from wkcuber.api.Properties.DatasetProperties import (
     WKProperties,
@@ -41,7 +43,7 @@ def convert_dtypes(
 
     # split the dtype into the actual type and the number of bits
     # example: "uint24" -> ["uint", "24"]
-    dtype_parts = re.split("(\d+)", str(dtype))
+    dtype_parts = re.split(r"(\d+)", str(dtype))
     # calculate number of bits for dtype_per_channel
     converted_dtype_parts = [
         (str(int(op(int(part), num_channels))) if is_int(part) else part)
@@ -76,16 +78,25 @@ def dtype_per_channel_to_dtype_per_layer(
     )
 
 
-class AbstractDataset(ABC):
+def copy_job(args: Tuple[View, View, int]) -> None:
+    (source_view, target_view, i) = args
+    # Copy the data form one view to the other in a buffered fashion
+    target_view.write(source_view.read())
+
+
+LayerT = TypeVar("LayerT", bound=Layer)
+
+
+class AbstractDataset(Generic[LayerT]):
     @abstractmethod
     def __init__(self, dataset_path: Union[str, Path]) -> None:
         properties: Properties = self._get_properties_type()._from_json(
             join(dataset_path, Properties.FILE_NAME)
         )
-        self.layers: Dict[str, Layer] = {}
+        self.layers: Dict[str, LayerT] = {}
         self.path = Path(properties.path).parent
         self.properties = properties
-        self.data_format = "abstract"
+        self._data_format = "abstract"
 
         # construct self.layer
         for layer_name in self.properties.data_layers:
@@ -126,7 +137,7 @@ class AbstractDataset(ABC):
     def get_properties(self) -> Properties:
         return self.properties
 
-    def get_layer(self, layer_name: str) -> Layer:
+    def get_layer(self, layer_name: str) -> LayerT:
         if layer_name not in self.layers.keys():
             raise IndexError(
                 "The layer {} is not a layer of this dataset".format(layer_name)
@@ -141,7 +152,7 @@ class AbstractDataset(ABC):
         dtype_per_channel: Union[str, np.dtype] = None,
         num_channels: int = None,
         **kwargs: Any,
-    ) -> Layer:
+    ) -> LayerT:
         if "dtype" in kwargs:
             raise ValueError(
                 f"Called Dataset.add_layer with 'dtype'={kwargs['dtype']}. This parameter is deprecated. Use 'dtype_per_layer' or 'dtype_per_channel' instead."
@@ -187,7 +198,7 @@ class AbstractDataset(ABC):
             layer_name,
             category,
             dtype_per_layer,
-            self.data_format,
+            self._data_format,
             num_channels,
             **kwargs,
         )
@@ -204,7 +215,7 @@ class AbstractDataset(ABC):
         dtype_per_channel: Union[str, np.dtype] = None,
         num_channels: int = None,
         **kwargs: Any,
-    ) -> Layer:
+    ) -> LayerT:
         if "dtype" in kwargs:
             raise ValueError(
                 f"Called Dataset.get_or_add_layer with 'dtype'={kwargs['dtype']}. This parameter is deprecated. Use 'dtype_per_layer' or 'dtype_per_channel' instead."
@@ -261,7 +272,7 @@ class AbstractDataset(ABC):
         # delete files on disk
         rmtree(join(self.path, layer_name))
 
-    def add_symlink_layer(self, foreign_layer_path: Union[str, Path]) -> Layer:
+    def add_symlink_layer(self, foreign_layer_path: Union[str, Path]) -> LayerT:
         foreign_layer_path = os.path.abspath(foreign_layer_path)
         layer_name = os.path.basename(os.path.normpath(foreign_layer_path))
         if layer_name in self.layers.keys():
@@ -292,21 +303,117 @@ class AbstractDataset(ABC):
     def get_view(
         self,
         layer_name: str,
-        mag: Union[str, Mag],
+        mag: Union[int, str, list, tuple, np.ndarray, Mag],
         size: Tuple[int, int, int],
         offset: Tuple[int, int, int] = None,
         is_bounded: bool = True,
+        read_only: bool = False,
     ) -> View:
         layer = self.get_layer(layer_name)
         mag_ds = layer.get_mag(mag)
 
-        return mag_ds.get_view(size=size, offset=offset, is_bounded=is_bounded)
+        return mag_ds.get_view(
+            size=size, offset=offset, is_bounded=is_bounded, read_only=read_only
+        )
 
     def _create_layer(
         self, layer_name: str, dtype_per_channel: np.dtype, num_channels: int
-    ) -> Layer:
+    ) -> LayerT:
         raise NotImplementedError
 
+    def copy_dataset(
+        self, empty_target_ds: "AbstractDataset", args: Optional[Namespace] = None
+    ) -> None:
+        assert (
+            len(empty_target_ds.layers) == 0
+        ), f"Copying dataset failed. The target dataset must be empty."
+        with get_executor_for_args(args) as executor:
+            for layer_name, layer in self.layers.items():
+                largest_segment_id = None
+                if (
+                    self.properties.data_layers[layer_name].category
+                    == Layer.SEGMENTATION_TYPE
+                ):
+                    largest_segment_id = self.properties.data_layers[
+                        layer_name
+                    ].largest_segment_id
+                target_layer = empty_target_ds.add_layer(
+                    layer_name,
+                    self.properties.data_layers[layer_name].category,
+                    dtype_per_channel=layer.dtype_per_channel,
+                    num_channels=layer.num_channels,
+                    largest_segment_id=largest_segment_id,
+                )
+
+                bbox = self.properties.get_bounding_box_of_layer(layer_name)
+
+                for mag_name, mag in layer.mags.items():
+                    target_mag = target_layer.add_mag(mag_name)
+
+                    # The bounding box needs to be updated manually because chunked views do not have a reference to the dataset itself
+                    # The base view of a MagDataset always starts at (0, 0, 0)
+                    target_mag.view.global_offset = (0, 0, 0)
+                    target_mag.view.size = cast(
+                        Tuple[int, int, int],
+                        tuple(
+                            BoundingBox(topleft=bbox[0], size=bbox[1])
+                            .align_with_mag(Mag(mag_name), ceil=True)
+                            .in_mag(Mag(mag_name))
+                            .bottomright
+                        ),
+                    )
+                    target_mag.layer.dataset.properties._set_bounding_box_of_layer(
+                        layer_name, offset=bbox[0], size=bbox[1]
+                    )
+
+                    # The data gets written to the target_mag.
+                    # Therefore, the chunk size is determined by the target_mag to prevent concurrent writes
+                    mag.view.for_zipped_chunks(
+                        work_on_chunk=copy_job,
+                        target_view=target_mag.view,
+                        source_chunk_size=target_mag._get_file_dimensions(),
+                        target_chunk_size=target_mag._get_file_dimensions(),
+                        executor=executor,
+                    )
+
+    def to_wk_dataset(
+        self,
+        new_dataset_path: Union[str, Path],
+        scale: Optional[Tuple[float, float, float]] = None,
+    ) -> "WKDataset":
+        if scale is None:
+            scale = self.properties.scale
+        new_ds = WKDataset.create(new_dataset_path, scale=scale)
+        self.copy_dataset(new_ds)
+        return new_ds
+
+    def to_tiff_dataset(
+        self,
+        new_dataset_path: Union[str, Path],
+        scale: Optional[Tuple[float, float, float]] = None,
+        pattern: Optional[str] = None,
+    ) -> "TiffDataset":
+        if scale is None:
+            scale = self.properties.scale
+        new_ds = TiffDataset.create(new_dataset_path, scale=scale, pattern=pattern)
+        self.copy_dataset(new_ds)
+        return new_ds
+
+    def to_tiled_tiff_dataset(
+        self,
+        new_dataset_path: Union[str, Path],
+        tile_size: Tuple[int, int],
+        scale: Optional[Tuple[float, float, float]] = None,
+        pattern: Optional[str] = None,
+    ) -> "TiledTiffDataset":
+        if scale is None:
+            scale = self.properties.scale
+        new_ds = TiledTiffDataset.create(
+            new_dataset_path, scale=scale, tile_size=tile_size, pattern=pattern
+        )
+        self.copy_dataset(new_ds)
+        return new_ds
+
     @abstractmethod
     def _get_properties_type(self) -> Type[Properties]:
         pass
@@ -316,7 +423,7 @@ class AbstractDataset(ABC):
         pass
 
 
-class WKDataset(AbstractDataset):
+class WKDataset(AbstractDataset[WKLayer]):
     @classmethod
     def create(
         cls, dataset_path: Union[str, Path], scale: Tuple[float, float, float]
@@ -342,15 +449,12 @@ class WKDataset(AbstractDataset):
 
     def __init__(self, dataset_path: Union[str, Path]) -> None:
         super().__init__(dataset_path)
-        self.data_format = "wkw"
+        self._data_format = "wkw"
         assert isinstance(self.properties, WKProperties)
 
-    def to_tiff_dataset(self, new_dataset_path: Union[str, Path]) -> "TiffDataset":
-        raise NotImplementedError  # TODO; implement
-
     def _create_layer(
         self, layer_name: str, dtype_per_channel: np.dtype, num_channels: int
-    ) -> Layer:
+    ) -> WKLayer:
         return WKLayer(layer_name, self, dtype_per_channel, num_channels)
 
     def _get_properties_type(self) -> Type[WKProperties]:
@@ -360,7 +464,7 @@ class WKDataset(AbstractDataset):
         return WKDataset
 
 
-class TiffDataset(AbstractDataset):
+class TiffDataset(AbstractDataset[TiffLayer]):
     properties: TiffProperties
 
     @classmethod
@@ -368,8 +472,10 @@ class TiffDataset(AbstractDataset):
         cls,
         dataset_path: Union[str, Path],
         scale: Tuple[float, float, float],
-        pattern: str = "{zzzzz}.tif",
+        pattern: Optional[str] = None,
     ) -> "TiffDataset":
+        if pattern is None:
+            pattern = "{zzzzz}.tif"
         validate_pattern(pattern)
         name = basename(normpath(dataset_path))
         properties = TiffProperties(
@@ -411,12 +517,9 @@ class TiffDataset(AbstractDataset):
         self.data_format = "tiff"
         assert isinstance(self.properties, TiffProperties)
 
-    def to_wk_dataset(self, new_dataset_path: Union[str, Path]) -> WKDataset:
-        raise NotImplementedError  # TODO; implement
-
     def _create_layer(
         self, layer_name: str, dtype_per_channel: np.dtype, num_channels: int
-    ) -> Layer:
+    ) -> TiffLayer:
         return TiffLayer(layer_name, self, dtype_per_channel, num_channels)
 
     def _get_properties_type(self) -> Type[TiffProperties]:
@@ -426,7 +529,7 @@ class TiffDataset(AbstractDataset):
         return TiffDataset
 
 
-class TiledTiffDataset(AbstractDataset):
+class TiledTiffDataset(AbstractDataset[TiledTiffLayer]):
     properties: TiffProperties
 
     @classmethod
@@ -435,8 +538,10 @@ class TiledTiffDataset(AbstractDataset):
         dataset_path: Union[str, Path],
         scale: Tuple[float, float, float],
         tile_size: Tuple[int, int],
-        pattern: str = "{xxxxx}/{yyyyy}/{zzzzz}.tif",
+        pattern: Optional[str] = None,
     ) -> "TiledTiffDataset":
+        if pattern is None:
+            pattern = "{xxxxx}/{yyyyy}/{zzzzz}.tif"
         validate_pattern(pattern)
         name = basename(normpath(dataset_path))
         properties = TiffProperties(
@@ -480,9 +585,6 @@ class TiledTiffDataset(AbstractDataset):
             else:
                 return cls.create(dataset_path, scale, tile_size, pattern)
 
-    def to_wk_dataset(self, new_dataset_path: str) -> WKDataset:
-        raise NotImplementedError  # TODO; implement
-
     def __init__(self, dataset_path: Union[str, Path]) -> None:
         super().__init__(dataset_path)
         self.data_format = "tiled_tiff"
@@ -490,7 +592,7 @@ class TiledTiffDataset(AbstractDataset):
 
     def _create_layer(
         self, layer_name: str, dtype_per_channel: np.dtype, num_channels: int
-    ) -> Layer:
+    ) -> TiledTiffLayer:
         return TiledTiffLayer(layer_name, self, dtype_per_channel, num_channels)
 
     def _get_properties_type(self) -> Type[TiffProperties]:
