@@ -94,7 +94,9 @@ def convert_knossos(
         with get_executor_for_args(args) as executor:
             knossos_cubes = list(source_knossos.list_cubes())
             if len(knossos_cubes) == 0:
-                logging.error("No input KNOSSOS cubes found.")
+                logging.error(
+                    "No input KNOSSOS cubes found. Make sure to pass the path which points to a KNOSSOS magnification (e.g., testdata/knossos/color/1)."
+                )
                 exit(1)
 
             knossos_cubes.sort()
@@ -105,10 +107,14 @@ def convert_knossos(
             wait_and_ensure_success(executor.map_to_futures(convert_cube_job, job_args))
 
 
+def main(args: Namespace) -> None:
+    convert_knossos(
+        args.source_path, args.target_path, args.layer_name, args.dtype, args.mag, args
+    )
+
+
 if __name__ == "__main__":
     args = create_parser().parse_args()
     setup_logging(args)
 
-    convert_knossos(
-        args.source_path, args.target_path, args.layer_name, args.dtype, args.mag, args
-    )
+    main(args)
