@@ -1,15 +1,16 @@
 import time
 import logging
-from typing import List, Tuple
+from typing import List, Tuple, Optional
 
 import numpy as np
 import wkw
 from argparse import ArgumentParser, Namespace
 from os import path
+from pathlib import Path
 from natsort import natsorted
 
 from .mag import Mag
-from .downsampling import (
+from .downsampling_utils import (
     parse_interpolation_mode,
     downsample_unpadded_data,
     InterpolationModes,
@@ -17,8 +18,8 @@ from .downsampling import (
 from .utils import (
     get_chunks,
     find_files,
-    add_verbose_flag,
     add_batch_size_flag,
+    add_verbose_flag,
     open_wkw,
     ensure_wkw,
     WkwDatasetInfo,
@@ -58,7 +59,7 @@ def create_parser() -> ArgumentParser:
         "--dtype",
         "-d",
         help="Target datatype (e.g. uint8, uint16, uint32)",
-        default="uint8",
+        default=None,
     )
 
     parser.add_argument(
@@ -95,10 +96,13 @@ def create_parser() -> ArgumentParser:
 
 
 def find_source_filenames(source_path: str) -> List[str]:
-    # Find all files in a folder that have a matching file extension
-    source_files = list(
-        find_files(path.join(source_path, "*"), image_reader.readers.keys())
-    )
+    # Find all source files that have a matching file extension
+
+    if Path(source_path).is_dir():
+        source_path = path.join(source_path, "*")
+
+    source_files = list(find_files(source_path, image_reader.readers.keys()))
+
     assert len(source_files) > 0, (
         "No image files found in path "
         + source_path
@@ -106,12 +110,13 @@ def find_source_filenames(source_path: str) -> List[str]:
         + str(image_reader.readers.keys())
         + "."
     )
+
     return natsorted(source_files)
 
 
-def read_image_file(file_name: str, dtype: type) -> np.ndarray:
+def read_image_file(file_name: str, dtype: type, z_slice: int) -> np.ndarray:
     try:
-        return image_reader.read_array(file_name, dtype)
+        return image_reader.read_array(file_name, dtype, z_slice)
     except Exception as exc:
         logging.error("Reading of file={} failed with {}".format(file_name, exc))
         raise exc
@@ -176,7 +181,7 @@ def cubing_job(
                 for z, file_name in zip(z_batch, source_file_batch):
                     # Image shape will be (x, y, channel_count, z=1)
                     image = read_image_file(
-                        file_name, target_wkw_info.header.voxel_type
+                        file_name, target_wkw_info.header.voxel_type, z
                     )
 
                     if not pad:
@@ -233,17 +238,28 @@ def cubing(
     source_path: str,
     target_path: str,
     layer_name: str,
-    dtype: str,
-    batch_size: int,
+    batch_size: Optional[int],
     args: Namespace,
 ) -> dict:
-
     source_files = find_source_filenames(source_path)
 
     # All images are assumed to have equal dimensions
     num_x, num_y = image_reader.read_dimensions(source_files[0])
     num_channels = image_reader.read_channel_count(source_files[0])
-    num_z = len(source_files)
+    num_z_slices_per_file = image_reader.read_z_slices_per_file(source_files[0])
+    assert (
+        num_z_slices_per_file == 1 or len(source_files) == 1
+    ), "Multi page TIFF support only for single files"
+    if num_z_slices_per_file > 1:
+        num_z = num_z_slices_per_file
+    else:
+        num_z = len(source_files)
+
+    if not hasattr(args, "dtype") or args.dtype is None:
+        args.dtype = image_reader.read_dtype(source_files[0])
+
+    if batch_size is None:
+        batch_size = BLOCK_LEN
 
     target_mag = Mag(args.target_mag)
     target_wkw_info = WkwDatasetInfo(
@@ -251,7 +267,7 @@ def cubing(
         layer_name,
         target_mag,
         wkw.Header(
-            convert_element_class_to_dtype(dtype),
+            convert_element_class_to_dtype(args.dtype),
             num_channels,
             file_len=args.wkw_file_len,
         ),
@@ -277,6 +293,11 @@ def cubing(
             # Prepare z batches
             max_z = min(num_z + start_z, z + BLOCK_LEN)
             z_batch = list(range(z, max_z))
+            # Prepare source files array
+            if len(source_files) > 1:
+                source_files_array = source_files[z - start_z : max_z - start_z]
+            else:
+                source_files_array = source_files * (max_z - z)
             # Prepare job
             job_args.append(
                 (
@@ -284,7 +305,7 @@ def cubing(
                     z_batch,
                     target_mag,
                     interpolation_mode,
-                    source_files[z - start_z : max_z - start_z],
+                    source_files_array,
                     batch_size,
                     (num_x, num_y),
                     args.pad,
@@ -305,7 +326,6 @@ if __name__ == "__main__":
         args.source_path,
         args.target_path,
         args.layer_name,
-        args.dtype,
         args.batch_size,
         args=args,
     )
